import os
from pylab import *
rc('font', size=12)

amgcl = loadtxt('dmem_data/lrz/amgcl_strong.txt', dtype={
        'names'   : ('size', 'omp', 'mpi', 'setup', 'solve', 'iters'),
        'formats' : ('i8', 'i4', 'i4', 'f8', 'f8', 'i4')
        })
trilinos = loadtxt('dmem_data/lrz/trilinos_strong.txt', dtype={
            'names'   : ('mpi', 'size', 'iters', 'setup', 'solve'),
            'formats' : ('i4', 'i8', 'i4', 'f8', 'f8')
            })

omp = unique(list(amgcl['omp']))

def set_ticks(ax):
    ax.set_xscale('log')
    ax.set_xticks([1, 7, 14, 28, 28 * 4, 28 * 16, 28 * 64])
    ax.get_xaxis().set_major_formatter(matplotlib.ticker.ScalarFormatter())
    ax.get_xaxis().set_tick_params(which='minor', size=0)
    ax.get_xaxis().set_tick_params(which='minor', width=0)

fig,gs = subplots(2, 2, figsize=(8,7))
handles = []

ax = gs[0,0]
for n in omp:
    I = (amgcl['omp'] == n)
    h = ax.loglog(n * amgcl[I]['mpi'], amgcl[I]['setup'] + amgcl[I]['solve'], 'o-')
    handles.append(h[0])
h = ax.loglog(trilinos['mpi'], trilinos['setup'] + trilinos['solve'], 's-')
handles.append(h[0])
c = trilinos['mpi']
t = trilinos['setup'][1] + trilinos['solve'][1]
h = ax.loglog(c, t * c[1] / c, 'k:')
handles.append(h[0])
set_ticks(ax)
ax.set_ylabel('Total time')

ax = gs[0,1]
for n in omp:
    I = (amgcl['omp'] == n)
    ax.loglog(n * amgcl[I]['mpi'], amgcl[I]['setup'], 'o-')
ax.loglog(trilinos['mpi'], trilinos['setup'], 's-')
t = trilinos['setup'][1]
h = ax.loglog(c, t * c[1] / c, 'k:')
handles.append(h[0])
set_ticks(ax)
ax.set_ylabel('Setup time')

ax = gs[1,0]
for n in omp:
    I = (amgcl['omp'] == n)
    ax.loglog(n * amgcl[I]['mpi'], amgcl[I]['solve'], 'o-')
ax.loglog(trilinos['mpi'], trilinos['solve'], 's-')
t = trilinos['solve'][1]
h = ax.loglog(c, t * c[1] / c, 'k:')
handles.append(h[0])
set_ticks(ax)
ax.set_ylabel('Solve time')

ax = gs[1,1]
for n in omp:
    I = (amgcl['omp'] == n)
    ax.semilogx(n * amgcl[I]['mpi'], amgcl[I]['iters'], 'o-')
ax.semilogx(trilinos['mpi'], trilinos['iters'], 's-')
ax.set_ylim([0, 30])
set_ticks(ax)
ax.set_ylabel('Iterations')

fig.tight_layout()

figlegend(handles,
    ['AMGCL, omp={}'.format(n) for n in omp] + ['Trilinos', 'Ideal'],
    ncol=3, loc='lower center')
fig.suptitle('Strong scaling of the Poisson problem on the SuperMUC cluster')
fig.subplots_adjust(top=0.93, bottom=0.15)

show()